import 'package:firebase_storage/firebase_storage.dart';
import 'package:flutter/material.dart';
import 'package:flutter_dropzone/flutter_dropzone.dart';
import 'package:uuid/uuid.dart';
import 'package:velocity_components/constants/_constants.dart';
import 'package:velocity_components/widgets/icon_border.dart';

class ImageUploadView extends StatefulWidget {
  final FirebaseStorage storageRef;
  final double aspectRatio;
  final String? imageUrl;
  final String directory;
  final Color borderColor;
  final Function(String) onImageUploaded;

  const ImageUploadView({
    Key? key,
    required this.storageRef,
    required this.onImageUploaded,
    this.aspectRatio = 2,
    this.imageUrl,
    this.directory = 'images',
    this.borderColor = darkBlue,
  }) : super(key: key);

  @override
  _ImageUploadViewState createState() => _ImageUploadViewState();
}

class _ImageUploadViewState extends State<ImageUploadView> {
  late DropzoneViewController _dropzoneViewController;
  Color _borderColor = darkBlue;
  String? _imageUrl;
  bool _loading = false;

  @override
  void initState() {
    super.initState();
    _imageUrl = widget.imageUrl;
    _borderColor = widget.borderColor;
  }

  _uploadFile(file) async {
    var fileType = await _dropzoneViewController.getFileMIME(file);

    if (fileType != 'image/jpeg' && fileType != 'image/png') {
      _showError('You can only upload .jpg and .png files');
      return;
    }

    this.setState(() {
      _loading = true;
    });

    final storageRef = widget.storageRef.ref(
        '${widget.directory}/${Uuid().v4()}.${fileType == 'image/png' ? 'png' : 'jpg'}');

    var uploadTaskSnapshot = await storageRef.putBlob(file);

    String downloadUrl = await uploadTaskSnapshot.ref.getDownloadURL();
    widget.onImageUploaded(downloadUrl);

    this.setState(() {
      _borderColor = widget.borderColor;
      _imageUrl = downloadUrl;
      _loading = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () async {
        try {
          final file = (await _dropzoneViewController.pickFiles())[0];
          _uploadFile(file);
        } catch (e) {
          print(e);
        }
      },
      child: Stack(
        children: [
          Positioned.fill(
            child: DropzoneView(
              operation: DragOperation.copy,
              cursor: CursorType.grab,
              onCreated: (ctrl) => _dropzoneViewController = ctrl,
              onError: (ev) {
                print('Error: $ev');
                setState(() {
                  _borderColor = widget.borderColor;
                });
              },
              onHover: () {
                this.setState(() {
                  _borderColor = green;
                });
              },
              onDrop: _uploadFile,
              onLeave: () {
                this.setState(() {
                  _borderColor = widget.borderColor;
                });
              },
            ),
          ),
          IconBorder(
            color: _borderColor,
            icon: Icons.camera_alt_rounded,
            aspectRatio: widget.aspectRatio,
            child: _loading
                ? Center(child: CircularProgressIndicator())
                : _imageUrl != null
                    ? Image.network(
                        _imageUrl!,
                        loadingBuilder: (context, widget, evt) =>
                            Center(child: Icon(Icons.camera_alt)),
                        fit: BoxFit.cover,
                      )
                    : Center(child: Icon(Icons.camera_alt)),
          ),
        ],
      ),
    );
  }

  _showError(message) {
    final snackBar = SnackBar(content: Text(message));
    ScaffoldMessenger.of(context).showSnackBar(snackBar);
  }
}
